@@ -68,12 +68,20 @@ def _get_workflow_execution_resource(wf_ex):
 
 # Use retries to prevent possible failures.
 @rest_utils.rest_retry_on_db_error
-def _get_workflow_execution(id, must_exist=True):
+def _get_workflow_execution(id, must_exist=True, fields=None):
+    if fields and 'id' not in fields:
+        fields.insert(0, 'id')
+
+    fields_tuple = rest_utils.fields_list_to_cls_fields_tuple(
+        db_models.WorkflowExecution,
+        fields
+    )
+
     with db_api.transaction():
         if must_exist:
-            wf_ex = db_api.get_workflow_execution(id)
+            wf_ex = db_api.get_workflow_execution(id, fields=fields_tuple)
         else:
-            wf_ex = db_api.load_workflow_execution(id)
+            wf_ex = db_api.load_workflow_execution(id, fields=fields_tuple)
 
         return rest_utils.load_deferred_fields(
             wf_ex,
@@ -90,19 +98,25 @@ class ExecutionsController(rest.RestController):
     executions = sub_execution.SubExecutionsController()
 
     @rest_utils.wrap_wsme_controller_exception
-    @wsme_pecan.wsexpose(resources.Execution, wtypes.text)
-    def get(self, id):
+    @wsme_pecan.wsexpose(resources.Execution, wtypes.text, types.uniquelist)
+    def get(self, id, fields=None):
         """Return the specified Execution.
 
         :param id: UUID of execution to retrieve.
+        :param fields: Optional. A specified list of fields of the resource to
+                       be returned. 'id' will be included automatically in
+                       fields if it's not provided.
         """
         acl.enforce("executions:get", context.ctx())
 
         LOG.debug("Fetch execution [id=%s]", id)
 
-        wf_ex = _get_workflow_execution(id)
+        wf_ex = _get_workflow_execution(id, fields=fields)
 
-        resource = resources.Execution.from_db_model(wf_ex)
+        if fields:
+            return resources.Execution.from_tuples(zip(fields, wf_ex))
+
+        resource = resources.Execution.from_db_model(wf_ex, fields=fields)
 
         resource.published_global = (
             data_flow.get_workflow_execution_published_global(wf_ex)
@@ -124,16 +138,22 @@ class ExecutionsController(rest.RestController):
         """
         acl.enforce('executions:update', context.ctx())
 
-        LOG.debug('Update execution [id=%s, execution=%s]', id, wf_ex)
+        LOG.info('Update execution [id=%s, execution=%s]', id, wf_ex)
 
         @rest_utils.rest_retry_on_db_error
         def _compute_delta(wf_ex):
             with db_api.transaction():
                 # ensure that workflow execution exists
-                db_api.get_workflow_execution(
+                wf_ex_old = db_api.get_workflow_execution(
                     id,
-                    fields=(db_models.WorkflowExecution.id,)
+                    fields=(db_models.WorkflowExecution.id,
+                            db_models.WorkflowExecution.root_execution_id)
                 )
+                root_execution_id = wf_ex_old.root_execution_id
+                if not root_execution_id:
+                    root_execution_id = wf_ex_old.id
+
+                context.ctx(root_execution_id=root_execution_id)
 
                 delta = {}
 
@@ -234,7 +254,7 @@ class ExecutionsController(rest.RestController):
         """
         acl.enforce('executions:create', context.ctx())
 
-        LOG.debug("Create execution [execution=%s]", wf_ex)
+        LOG.info("Create execution [execution=%s]", wf_ex)
 
         exec_dict = wf_ex.to_dict()
 
@@ -242,6 +262,7 @@ class ExecutionsController(rest.RestController):
 
         if not exec_id:
             exec_id = uuidutils.generate_uuid()
+            context.ctx(root_execution_id=exec_id)
 
             LOG.debug("Generated execution id [exec_id=%s]", exec_id)
 
@@ -253,6 +274,7 @@ class ExecutionsController(rest.RestController):
             # If yes, the method just returns the object. If not, the ID
             # will be used to create a new execution.
             wf_ex = _get_workflow_execution(exec_id, must_exist=False)
+            context.ctx(root_execution_id=exec_id)
 
             if wf_ex:
                 return resources.Execution.from_db_model(wf_ex)
@@ -363,7 +385,7 @@ class ExecutionsController(rest.RestController):
                           or less than that of sort_keys.
         :param fields: Optional. A specified list of fields of the resource to
                        be returned. 'id' will be included automatically in
-                       fields if it's provided, since it will be used when
+                       fields if it's not provided, since it will be used when
                        constructing 'next' link.
         :param workflow_name: Optional. Keep only resources with a specific
                               workflow name.
