@@ -12,6 +12,15 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
+# NOTE(amorin) Deprecated wsgi entry point
+# use wsgi/ instead now
+
+# NOTE(amorin)
+# Hardcode the threading backend to avoid using eventlet until this will
+# eventually become the default
+import oslo_service.backend as service_backend
+service_backend.init_backend(service_backend.BackendType.THREADING)
+
 from mistral.api import app
 
 application = app.init_wsgi()
