@@ -21,12 +21,13 @@ Configuration options registration and useful routines.
 """
 
 import json
+import os
 
 from keystoneauth1 import loading
 from oslo_config import cfg
+from oslo_config import types
 from oslo_log import log
 from oslo_middleware import cors
-from oslo_service import _options as service_opts
 from osprofiler import opts as profiler
 
 from mistral import version
@@ -38,9 +39,10 @@ from mistral.workflow import states
 launch_opt = cfg.ListOpt(
     'server',
     default=['all'],
-    help=_('Specifies which mistral server to start by the launch script. '
-           'Valid options are all or any combination of '
-           'api, engine, and executor.')
+    item_type=types.String(
+        choices=('all', 'api', 'engine', 'executor', 'event-engine',
+                 'notifier')),
+    help=_('Specifies which mistral server to start by the launch script.')
 )
 
 wf_trace_log_name_opt = cfg.StrOpt(
@@ -49,21 +51,35 @@ wf_trace_log_name_opt = cfg.StrOpt(
     help=_('Logger name for pretty workflow trace output.')
 )
 
-use_debugger_opt = cfg.BoolOpt(
-    'use-debugger',
-    default=False,
-    help=_('Enables debugger. Note that using this option changes how the '
-           'eventlet library is used to support async IO. This could result '
-           'in failures that do not occur under normal operation. '
-           'Use at your own risk.')
-)
-
 auth_type_opt = cfg.StrOpt(
     'auth_type',
     default='keystone',
     help=_('Authentication type (valid options: keystone, keycloak-oidc)')
 )
 
+action_providers_opts = [
+    cfg.ListOpt(
+        'allowlist',
+        default=[],
+        help=_(
+            'Allowlist with action providers that is allowed to be '
+            'loaded from the entry point "mistral.action.providers", '
+            'if empty all action providers will be allowed unless '
+            'denylist is set.'
+        ),
+    ),
+    cfg.ListOpt(
+        'denylist',
+        default=[],
+        help=_(
+            'Denylist with action providers that is not allowed to '
+            'be loaded from the entry point "mistral.action.providers", '
+            'allowlist takes precendence, if empty all action providers '
+            'will be allowed.'
+        ),
+    ),
+]
+
 legacy_action_provider_opts = [
     cfg.BoolOpt(
         'load_action_plugins',
@@ -91,6 +107,25 @@ legacy_action_provider_opts = [
             'This property is needed mostly for testing.'
         )
     ),
+    cfg.ListOpt(
+        'allowlist',
+        default=[],
+        help=_(
+            'Allowlist with actions that is allowed to be '
+            'loaded from the entry point "mistral.actions", '
+            'if empty all actions will be allowed.'
+        ),
+    ),
+    cfg.ListOpt(
+        'denylist',
+        default=[],
+        help=_(
+            'Denylist with actions that is not allowed to '
+            'be loaded from the entry point "mistral.actions", '
+            'allowlist takes precedence, if empty all actions '
+            'will be allowed.'
+        ),
+    ),
 ]
 
 api_opts = [
@@ -114,6 +149,11 @@ api_opts = [
     ),
     cfg.IntOpt(
         'api_workers',
+        deprecated_for_removal=True,
+        deprecated_reason='This option has become useless and mistral-api'
+                          'will start only one worker now. If you want '
+                          'more workers, consider starting mistral-wsgi-api '
+                          'with a wsgi server instead.',
         help=_('Number of workers for Mistral API service '
                'default is equal to the number of CPUs available if that can '
                'be determined, else a default worker count of 1 is returned.')
@@ -151,27 +191,6 @@ js_impl_opt = cfg.StrOpt(
            'action to evaluate scripts.')
 )
 
-rpc_impl_opt = cfg.StrOpt(
-    'rpc_implementation',
-    default='oslo',
-    choices=['oslo', 'kombu'],
-    deprecated_for_removal=True,
-    deprecated_reason='Kombu driver is deprecated and will be removed '
-                      'in the F release cycle',
-    help=_('Specifies RPC implementation for RPC client and server. '
-           'Support of kombu driver is experimental.')
-)
-
-oslo_rpc_executor = cfg.StrOpt(
-    'oslo_rpc_executor',
-    default='threading',
-    choices=['eventlet', 'threading'],
-    deprecated_for_removal=True,
-    deprecated_reason='This option is going to be removed from oslo.messaging',
-    help=_('Executor type used by Oslo Messaging framework. Defines how '
-           'Oslo Messaging based RPC subsystem processes incoming calls.')
-)
-
 expiration_token_duration = cfg.IntOpt(
     'expiration_token_duration',
     default=30,
@@ -590,23 +609,6 @@ context_versioning_opts = [
     )
 ]
 
-coordination_opts = [
-    cfg.StrOpt(
-        'backend_url',
-        secret=True,
-        deprecated_for_removal=True,
-        deprecated_reason='Coordination will be removed from mistral code',
-        help=_('The backend URL to be used for coordination')
-    ),
-    cfg.FloatOpt(
-        'heartbeat_interval',
-        default=5.0,
-        deprecated_for_removal=True,
-        deprecated_reason='This option has been unused and has had no effect',
-        help=_('Number of seconds between heartbeats for coordination.')
-    )
-]
-
 profiler_opts = profiler.list_opts()[0][1]
 profiler_opts.append(
     cfg.StrOpt(
@@ -755,6 +757,7 @@ healthcheck_opts = [
 
 CONF = cfg.CONF
 
+ACTION_PROVIDERS_GROUP = 'action_providers'
 LEGACY_ACTION_PROVIDER_GROUP = 'legacy_action_provider'
 API_GROUP = 'api'
 ENGINE_GROUP = 'engine'
@@ -764,7 +767,6 @@ CRON_TRIGGER_GROUP = 'cron_trigger'
 EVENT_ENGINE_GROUP = 'event_engine'
 NOTIFIER_GROUP = 'notifier'
 PECAN_GROUP = 'pecan'
-COORDINATION_GROUP = 'coordination'
 EXECUTION_EXPIRATION_POLICY_GROUP = 'execution_expiration_policy'
 ACTION_HEARTBEAT_GROUP = 'action_heartbeat'
 ACTION_LOGGING_GROUP = 'action_logging'
@@ -780,11 +782,9 @@ CONF.register_opt(wf_trace_log_name_opt)
 CONF.register_opt(auth_type_opt)
 CONF.register_opt(scheduler_type_opt)
 CONF.register_opt(js_impl_opt)
-CONF.register_opt(rpc_impl_opt)
-CONF.register_opt(oslo_rpc_executor)
 CONF.register_opt(expiration_token_duration)
-CONF.register_opts(service_opts.service_opts)
 
+CONF.register_opts(action_providers_opts, group=ACTION_PROVIDERS_GROUP)
 CONF.register_opts(
     legacy_action_provider_opts,
     group=LEGACY_ACTION_PROVIDER_GROUP
@@ -807,7 +807,6 @@ CONF.register_opts(context_versioning_opts, group=CONTEXT_VERSIONING_GROUP)
 CONF.register_opts(event_engine_opts, group=EVENT_ENGINE_GROUP)
 CONF.register_opts(notifier_opts, group=NOTIFIER_GROUP)
 CONF.register_opts(pecan_opts, group=PECAN_GROUP)
-CONF.register_opts(coordination_opts, group=COORDINATION_GROUP)
 CONF.register_opts(profiler_opts, group=PROFILER_GROUP)
 CONF.register_opts(keycloak_oidc_opts, group=KEYCLOAK_OIDC_GROUP)
 CONF.register_opts(yaql_opts, group=YAQL_GROUP)
@@ -815,7 +814,6 @@ CONF.register_opts(healthcheck_opts, group=HEALTHCHECK_GROUP)
 loading.register_session_conf_options(CONF, KEYSTONE_GROUP)
 
 CLI_OPTS = [
-    use_debugger_opt,
     launch_opt
 ]
 
@@ -824,8 +822,6 @@ default_group_opts = CLI_OPTS + [
     auth_type_opt,
     scheduler_type_opt,
     js_impl_opt,
-    rpc_impl_opt,
-    oslo_rpc_executor,
     expiration_token_duration
 ]
 
@@ -842,6 +838,8 @@ _DEFAULT_LOG_LEVELS = [
 
 def list_opts():
     return [
+        (ACTION_PROVIDERS_GROUP, action_providers_opts),
+        (LEGACY_ACTION_PROVIDER_GROUP, legacy_action_provider_opts),
         (API_GROUP, api_opts),
         (ENGINE_GROUP, engine_opts),
         (EXECUTOR_GROUP, executor_opts),
@@ -850,7 +848,6 @@ def list_opts():
         (CRON_TRIGGER_GROUP, cron_trigger_opts),
         (NOTIFIER_GROUP, notifier_opts),
         (PECAN_GROUP, pecan_opts),
-        (COORDINATION_GROUP, coordination_opts),
         (EXECUTION_EXPIRATION_POLICY_GROUP, execution_expiration_policy_opts),
         (PROFILER_GROUP, profiler_opts),
         (KEYCLOAK_OIDC_GROUP, keycloak_oidc_opts),
@@ -863,7 +860,13 @@ def list_opts():
     ]
 
 
-def parse_args(args=None, usage=None, default_config_files=None):
+def parse_args(args=None, usage=None):
+    # NOTE(amorin) allow overwritting config dir and file from env
+    conf_d = os.environ.get('MISTRAL_CONFIG_DIR')
+    default_config_dirs = [conf_d] if conf_d else None
+    conf_f = os.environ.get('MISTRAL_CONFIG_FILE')
+    default_config_files = [conf_f] if conf_f else None
+
     default_log_levels = log.get_default_log_levels()
     default_log_levels.extend(_DEFAULT_LOG_LEVELS)
     log.set_defaults(default_log_levels=default_log_levels)
@@ -875,7 +878,8 @@ def parse_args(args=None, usage=None, default_config_files=None):
         project='mistral',
         version=version.version_string,
         usage=usage,
-        default_config_files=default_config_files
+        default_config_dirs=default_config_dirs,
+        default_config_files=default_config_files,
     )
 
 
