@@ -13,6 +13,12 @@
 
 """Starter script for mistral-db-manage."""
 
+# NOTE(amorin)
+# Hardcode the threading backend to avoid using eventlet until this will
+# eventually become the default
+import oslo_service.backend as service_backend
+service_backend.init_backend(service_backend.BackendType.THREADING)
+
 import os
 
 from alembic import command as alembic_cmd
