@@ -15,8 +15,8 @@
 from oslo_db.sqlalchemy import models as oslo_models
 import sqlalchemy as sa
 from sqlalchemy import event
-from sqlalchemy.ext import declarative
 from sqlalchemy.orm import attributes
+from sqlalchemy.orm import declarative_base
 
 from mistral.services import security
 from mistral_lib import utils
@@ -87,15 +87,18 @@ class _MistralModelBase(oslo_models.ModelBase, oslo_models.TimestampMixin):
             if col.name not in unloaded and hasattr(self, col.name):
                 yield col.name
 
-    def iter_columns(self):
+    def iter_columns(self, fields=()):
         """Returns an iterator for loaded columns.
 
+        :param fields: names of fields to return
+        :type fields: tuple, list or set
         :return: A generator function that generates
             tuples (column name, column value).
         """
 
         for col_name in self.iter_column_names():
-            yield col_name, getattr(self, col_name)
+            if not fields or col_name in fields:
+                yield col_name, getattr(self, col_name)
 
     def get_clone(self):
         """Clones current object, loads all fields and returns the result."""
@@ -146,7 +149,7 @@ class _MistralModelBase(oslo_models.ModelBase, oslo_models.TimestampMixin):
                 )
 
 
-MistralModelBase = declarative.declarative_base(cls=_MistralModelBase)
+MistralModelBase = declarative_base(cls=_MistralModelBase)
 
 
 # Secure model related stuff.
