@@ -1,5 +1,6 @@
 # Copyright 2015 - Mirantis, Inc.
 # Copyright 2016 - Brocade Communications Systems, Inc.
+# Copyright 2020 Nokia Software.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -21,6 +22,7 @@ from mistral.db.v2.sqlalchemy import models
 from mistral.engine import actions
 from mistral.engine import task_handler
 from mistral import exceptions as exc
+from mistral.services import actions as action_service
 
 
 LOG = logging.getLogger(__name__)
@@ -81,25 +83,35 @@ def on_action_update(action_ex, state):
 @profiler.trace('action-handler-build-action', hide_args=True)
 def _build_action(action_ex):
     if isinstance(action_ex, models.WorkflowExecution):
-        return actions.WorkflowAction(wf_name=action_ex.name,
-                                      action_ex=action_ex)
+        return actions.WorkflowAction(
+            wf_name=action_ex.name,
+            action_ex=action_ex
+        )
 
-    adhoc_action_name = action_ex.runtime_context.get('adhoc_action_name')
+    action_desc = action_service.get_system_action_provider().find(
+        action_ex.name,
+        action_ex.workflow_namespace
+    )
 
-    if adhoc_action_name:
-        action_def = actions.resolve_action_definition(adhoc_action_name)
+    if action_desc is None:
+        raise exc.InvalidActionException(
+            "Failed to find action [action_name=%s]" %
+            action_ex.name
+        )
 
-        return actions.AdHocAction(action_def, action_ex=action_ex)
-
-    action_def = actions.resolve_action_definition(action_ex.name)
-
-    return actions.PythonAction(action_def, action_ex=action_ex)
+    return actions.RegularAction(action_desc, action_ex)
 
 
-def build_action_by_name(action_name):
-    action_def = actions.resolve_action_definition(action_name)
+def build_action_by_name(action_name, namespace=''):
+    action_desc = action_service.get_system_action_provider().find(
+        action_name,
+        namespace=namespace
+    )
 
-    action_cls = (actions.PythonAction if not action_def.spec
-                  else actions.AdHocAction)
+    if action_desc is None:
+        raise exc.InvalidActionException(
+            "Failed to find action [action_name=%s]" %
+            action_name
+        )
 
-    return action_cls(action_def)
+    return actions.RegularAction(action_desc)
