@@ -44,12 +44,11 @@ class EngineServer(service_base.MistralService):
     """Engine server.
 
     This class manages engine life-cycle and gets registered as an RPC
-    endpoint to process engine specific calls. It also registers a
-    cluster member associated with this instance of engine.
+    endpoint to process engine specific calls.
     """
 
     def __init__(self, engine, setup_profiler=True):
-        super(EngineServer, self).__init__('engine_group', setup_profiler)
+        super(EngineServer, self).__init__(setup_profiler)
 
         self.engine = engine
         self._rpc_server = None
@@ -88,24 +87,12 @@ class EngineServer(service_base.MistralService):
         self._rpc_server = rpc.get_rpc_server_driver()(CONF.engine)
         self._rpc_server.register_endpoint(self)
 
-        self._rpc_server.run(executor=CONF.oslo_rpc_executor)
+        self._rpc_server.run(executor='threading')
 
         self._notify_started('Engine server started.')
 
     def stop(self, graceful=False):
-        # NOTE(rakhmerov): Unfortunately, oslo.service doesn't pass the
-        # 'graceful' parameter with a correct value. It's simply ignored
-        # in the corresponding call chain leading to a concrete service.
-        # The only workaround for now is to check 'graceful_shutdown_timeout'
-        # configuration option. If it's not empty (not None or 0) then we
-        # should treat it a graceful shutdown.
-        graceful = bool(CONF.graceful_shutdown_timeout)
-
-        LOG.info(
-            'Stopping an engine server [graceful=%s, timeout=%s]',
-            graceful,
-            CONF.graceful_shutdown_timeout
-        )
+        LOG.info('Stopping engine server')
 
         super(EngineServer, self).stop(graceful)
 
