@@ -22,7 +22,6 @@ import threading
 
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_service import threadgroup
 
 from mistral import context as auth_ctx
 from mistral.db.v2 import api as db_api
@@ -138,7 +137,10 @@ class DefaultEventEngine(base.EventEngine):
     def __init__(self):
         self.engine_client = rpc.get_engine_client()
         self.event_queue = queue.Queue()
-        self.handler_tg = threadgroup.ThreadGroup()
+
+        self._stopped = False
+        self._thread = threading.Thread(target=self._loop)
+        self._thread.daemon = True
 
         self.event_triggers_map = defaultdict(list)
         self.exchange_topic_events_map = defaultdict(set)
@@ -147,12 +149,20 @@ class DefaultEventEngine(base.EventEngine):
         self.lock = threading.Lock()
 
         LOG.debug('Loading notification definitions.')
-
         self.notification_converter = NotificationsConverter()
 
-        self._start_handler()
+    def start(self):
+        LOG.info('Starting event notification engine...')
+        self._thread.start()
         self._start_listeners()
 
+    def stop(self):
+        self._stopped = True
+        self._thread.join()
+        for listener in self.exchange_topic_listener_map.values():
+            listener.stop()
+            listener.wait()
+
     def _get_endpoint_cls(self, events):
         """Create a messaging endpoint class.
 
@@ -205,11 +215,6 @@ class DefaultEventEngine(base.EventEngine):
 
         self.exchange_topic_listener_map[key] = listener
 
-    def stop_all_listeners(self):
-        for listener in self.exchange_topic_listener_map.values():
-            listener.stop()
-            listener.wait()
-
     def _start_listeners(self):
         triggers = db_api.get_event_triggers(insecure=True)
 
@@ -266,63 +271,65 @@ class DefaultEventEngine(base.EventEngine):
             finally:
                 auth_ctx.set_ctx(None)
 
-    def _process_event_queue(self, *args, **kwargs):
+    def _loop(self, *args, **kwargs):
         """Process notification events.
 
         This function is called in a thread.
         """
-        while True:
-            event = self.event_queue.get()
+        while not self._stopped:
+            try:
+                # Get from queue (nowait)
+                # It may raise a queue.Empty if there is nothing in queue
+                # but thanks to while loop we will continue getting until
+                # the thread got stopped.
+                event = self.event_queue.get_nowait()
+                context = event.get('context')
+                event_type = event.get('event_type')
 
-            context = event.get('context')
-            event_type = event.get('event_type')
+                # NOTE(kong): Use lock here to protect event_triggers_map
+                # variable from being updated outside the thread.
+                with self.lock:
+                    if event_type in self.event_triggers_map:
+                        triggers = self.event_triggers_map[event_type]
 
-            # NOTE(kong): Use lock here to protect event_triggers_map variable
-            # from being updated outside the thread.
-            with self.lock:
-                if event_type in self.event_triggers_map:
-                    triggers = self.event_triggers_map[event_type]
+                        # There may be more projects registered the same event.
+                        project_ids = [t['project_id'] for t in triggers]
 
-                    # There may be more projects registered the same event.
-                    project_ids = [t['project_id'] for t in triggers]
-
-                    any_public = any(
-                        [t['scope'] == 'public' for t in triggers]
-                    )
-
-                    # Skip the event doesn't belong to any event trigger owner.
-                    if (not any_public and CONF.pecan.auth_enable and
-                            context.get('project_id', '') not in project_ids):
-                        self.event_queue.task_done()
-                        continue
-
-                    # Need to choose what trigger(s) should be called exactly.
-                    triggers_to_call = []
-                    for t in triggers:
-                        project_trigger = (
-                            t['project_id'] == context.get('project_id')
+                        any_public = any(
+                            [t['scope'] == 'public' for t in triggers]
                         )
-                        public_trigger = t['scope'] == 'public'
-                        if project_trigger or public_trigger:
-                            triggers_to_call.append(t)
 
-                    LOG.debug('Start to handle event: %s, %d trigger(s) '
-                              'registered.', event_type, len(triggers))
+                        # Skip the event doesn't belong to any event trigger
+                        # owner.
+                        if (not any_public and CONF.pecan.auth_enable and
+                                context.get('project_id', '')
+                                not in project_ids):
+                            self.event_queue.task_done()
+                            continue
 
-                    event_params = self.notification_converter.convert(
-                        event_type,
-                        event
-                    )
+                        # Need to choose what trigger(s) should be called
+                        # exactly.
+                        triggers_to_call = []
+                        for t in triggers:
+                            project_trigger = (
+                                t['project_id'] == context.get('project_id')
+                            )
+                            public_trigger = t['scope'] == 'public'
+                            if project_trigger or public_trigger:
+                                triggers_to_call.append(t)
 
-                    self._start_workflow(triggers_to_call, event_params)
+                        LOG.debug('Start to handle event: %s, %d trigger(s) '
+                                  'registered.', event_type, len(triggers))
 
-            self.event_queue.task_done()
+                        event_params = self.notification_converter.convert(
+                            event_type,
+                            event
+                        )
 
-    def _start_handler(self):
-        """Starts event queue handler in a thread group."""
-        LOG.info('Starting event notification task...')
-
-        self.handler_tg.add_thread(self._process_event_queue)
+                        self._start_workflow(triggers_to_call, event_params)
+                self.event_queue.task_done()
+            except queue.Empty:
+                pass
 
     def process_notification_event(self, notification):
         """Callback function by event handler.
