@@ -29,13 +29,11 @@ class EventEngineServer(service_base.MistralService):
     """RPC EventEngine server.
 
     This class manages event engine life-cycle and gets registered as
-    an RPC endpoint to process event engine specific calls. It also
-    registers a cluster member associated with this instance of event
-    engine.
+    an RPC endpoint to process event engine specific calls.
     """
 
     def __init__(self, event_engine):
-        super(EventEngineServer, self).__init__('event_engine_group')
+        super(EventEngineServer, self).__init__()
 
         self._event_engine = event_engine
         self._rpc_server = None
@@ -43,6 +41,8 @@ class EventEngineServer(service_base.MistralService):
     def start(self):
         super(EventEngineServer, self).start()
 
+        self._event_engine.start()
+
         profiler_utils.setup(
             'mistral-event-engine',
             cfg.CONF.event_engine.host
@@ -63,6 +63,8 @@ class EventEngineServer(service_base.MistralService):
         if self._rpc_server:
             self._rpc_server.stop(graceful)
 
+        self._event_engine.stop()
+
     def create_event_trigger(self, rpc_ctx, trigger, events):
         LOG.info(
             "Received RPC request 'create_event_trigger'[rpc_ctx=%s,"
