@@ -13,34 +13,87 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import six
-
 from mistral.lang import types
 from mistral.lang.v2 import base
 from mistral.lang.v2 import publish
 
+NEXT_TASK = {
+    "oneOf": [
+        {
+            "type": "string",
+            "pattern": r"^\S+$",
+            "description": "Task name (e.g.: `task1`)"
+        },
+        {
+            "type": "string",
+            "pattern": r"^\w+ \w+=(.*)$",
+            "description": "Task name with dict parameter "
+                           "(e.g.: `fail msg=\"test\"`, "
+                           "`fail msg=<% task() %>`)"
+        },
+        {
+            "type": "string",
+            "pattern": r"^\w+\(\w+=(.*)\)$",
+            "description": "Task name with func parameter "
+                           "(e.g.: `fail(msg=\"test\")`, "
+                           "`fail(msg=<% task() %>)`)"
+        }
+    ]
+}
+
+TASK_WITH_EXPRESSION = {
+    "type": "object",
+    "minProperties": 1,
+    "maxProperties": 1,
+    "patternProperties": {
+        x['pattern']: types.EXPRESSION for x in NEXT_TASK['oneOf']
+    },
+    "description": "All next task variants plus expression (e.g.: "
+                   "`task1: <% $.vm_id != null %>`, "
+                   "`fail(msg=\"test\"): <% $.vm_id != null %>`)."
+}
+
+LIST_OF_TASKS = {
+    "type": "array",
+    "items": {
+        "oneOf": [
+            NEXT_TASK,
+            TASK_WITH_EXPRESSION
+        ]
+    },
+    "uniqueItems": True,
+    "minItems": 1
+}
+
+ADVANCED_PUBLISHING_DICT = {
+    "type": "object",
+    "minProperties": 1,
+    "properties": {
+        "publish": publish.PublishSpec.get_schema(),
+        "next": {
+            "oneOf": [
+                NEXT_TASK,
+                TASK_WITH_EXPRESSION,
+                LIST_OF_TASKS
+            ]
+        }
+    },
+    "additionalProperties": False
+}
+
 
 class OnClauseSpec(base.BaseSpec):
-    _simple_schema = {
+    _schema = {
         "oneOf": [
-            types.NONEMPTY_STRING,
-            types.UNIQUE_STRING_OR_EXPRESSION_CONDITION_LIST
+            NEXT_TASK,
+            TASK_WITH_EXPRESSION,
+            LIST_OF_TASKS,
+            ADVANCED_PUBLISHING_DICT
         ]
     }
 
-    _advanced_schema = {
-        "type": "object",
-        "properties": {
-            "publish": types.NONEMPTY_DICT,
-            "next": _simple_schema,
-        },
-        "additionalProperties": False
-    }
-
-    _schema = {"oneOf": [_simple_schema, _advanced_schema]}
-
-    def __init__(self, data):
-        super(OnClauseSpec, self).__init__(data)
+    def __init__(self, data, validate):
+        super(OnClauseSpec, self).__init__(data, validate)
 
         if not isinstance(data, dict):
             # Old simple schema.
@@ -52,7 +105,7 @@ class OnClauseSpec(base.BaseSpec):
             self._next = prepare_next_clause(data.get('next'))
 
     @classmethod
-    def get_schema(cls, includes=['definitions']):
+    def get_schema(cls, includes=('definitions',)):
         return super(OnClauseSpec, cls).get_schema(includes)
 
     def get_publish(self):
@@ -66,7 +119,7 @@ def _as_list_of_tuples(data):
     if not data:
         return []
 
-    if isinstance(data, six.string_types):
+    if isinstance(data, str):
         return [_as_tuple(data)]
 
     return [_as_tuple(item) for item in data]
