@@ -12,9 +12,9 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
+from http import HTTPStatus
 import json
 import requests
-from six.moves import http_client
 
 from oslo_log import log as logging
 
@@ -26,7 +26,7 @@ LOG = logging.getLogger(__name__)
 
 class WebhookPublisher(base.NotificationPublisher):
 
-    def publish(self, ex_id, data, event, timestamp, **kwargs):
+    def publish(self, ctx, ex_id, data, event, timestamp, **kwargs):
         url = kwargs.get('url')
         headers = kwargs.get('headers', {})
 
@@ -34,5 +34,5 @@ class WebhookPublisher(base.NotificationPublisher):
 
         LOG.info("Webook request url=%s code=%s", url, resp.status_code)
 
-        if resp.status_code not in [http_client.OK, http_client.CREATED]:
+        if resp.status_code not in [HTTPStatus.OK, HTTPStatus.CREATED]:
             raise Exception(resp.text)
