@@ -2,6 +2,7 @@
 # Copyright 2015 - StackStorm, Inc.
 # Copyright 2017 - Brocade Communications Systems, Inc.
 # Copyright 2018 - Extreme Networks, Inc.
+# Copyright 2020 Nokia Software.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -137,6 +138,15 @@ class EngineClient(eng.Engine):
 
         call = self._client.async_call if async_ else self._client.sync_call
 
+        LOG.info(
+            "Send RPC request 'start_workflow'[workflow_identifier=%s, "
+            "workflow_input=%s, description=%s, params=%s]",
+            wf_identifier,
+            wf_input,
+            description,
+            params
+        )
+
         return call(
             auth_ctx.ctx(),
             'start_workflow',
@@ -148,14 +158,34 @@ class EngineClient(eng.Engine):
             params=params
         )
 
+    @base.wrap_messaging_exception
+    def start_task(self, task_ex_id, first_run, waiting,
+                   triggered_by, rerun, reset, **params):
+        """Starts task sending a request to engine over RPC.
+
+        """
+
+        return self._client.async_call(
+            auth_ctx.ctx(),
+            'start_task',
+            task_ex_id=task_ex_id,
+            first_run=first_run,
+            waiting=waiting,
+            triggered_by=triggered_by,
+            rerun=rerun,
+            reset=reset,
+            params=params
+        )
+
     @base.wrap_messaging_exception
     def start_action(self, action_name, action_input,
-                     description=None, **params):
+                     description=None, namespace='', **params):
         """Starts action sending a request to engine over RPC.
 
         :param action_name: Action name.
         :param action_input: Action input data as a dictionary.
         :param description: Execution description.
+        :param namespace: The namespace of the action.
         :param params: Additional options for action running.
         :return: Action execution.
         """
@@ -165,6 +195,7 @@ class EngineClient(eng.Engine):
             action_name=action_name,
             action_input=action_input or {},
             description=description,
+            namespace=namespace,
             params=params
         )
 
@@ -196,6 +227,13 @@ class EngineClient(eng.Engine):
 
         call = self._client.async_call if async_ else self._client.sync_call
 
+        LOG.info(
+            "Send RPC request 'on_action_complete'[action_ex_id=%s, "
+            "result=%s]",
+            action_ex_id,
+            result.cut_repr() if result else None
+        )
+
         return call(
             auth_ctx.ctx(),
             'on_action_complete',
@@ -218,7 +256,7 @@ class EngineClient(eng.Engine):
         on_action_complete if the action execution reached completion state.
 
         :param action_ex_id: Action execution id.
-        :param action_ex_id: Updated state.
+        :param state: Updated state.
         :param wf_action: If True it means that the given id points to
             a workflow execution rather than action execution. It happens
             when a nested workflow execution sends its result to a parent
@@ -230,6 +268,13 @@ class EngineClient(eng.Engine):
 
         call = self._client.async_call if async_ else self._client.sync_call
 
+        LOG.info(
+            "Send RPC request 'on_action_update'"
+            "[action_ex_id=%s, state=%s]",
+            action_ex_id,
+            state
+        )
+
         return call(
             auth_ctx.ctx(),
             'on_action_update',
@@ -246,6 +291,11 @@ class EngineClient(eng.Engine):
         :return: Workflow execution.
         """
 
+        LOG.info(
+            "Send RPC request 'pause_workflow'[execution_id=%s]",
+            wf_ex_id
+        )
+
         return self._client.sync_call(
             auth_ctx.ctx(),
             'pause_workflow',
@@ -253,7 +303,7 @@ class EngineClient(eng.Engine):
         )
 
     @base.wrap_messaging_exception
-    def rerun_workflow(self, task_ex_id, reset=True, env=None):
+    def rerun_workflow(self, task_ex_id, reset=True, skip=False, env=None):
         """Rerun the workflow.
 
         This method reruns workflow with the given execution id
@@ -262,15 +312,23 @@ class EngineClient(eng.Engine):
         :param task_ex_id: Task execution id.
         :param reset: If true, then reset task execution state and purge
             action execution for the task.
+        :param skip: If True, then skip failed task and continue workflow
+            execution.
         :param env: Environment variables to update.
         :return: Workflow execution.
         """
 
+        LOG.info(
+            "Send RPC request 'rerun_workflow'[task_ex_id=%s]",
+            task_ex_id
+        )
+
         return self._client.sync_call(
             auth_ctx.ctx(),
             'rerun_workflow',
             task_ex_id=task_ex_id,
             reset=reset,
+            skip=skip,
             env=env
         )
 
@@ -283,6 +341,11 @@ class EngineClient(eng.Engine):
         :return: Workflow execution.
         """
 
+        LOG.info(
+            "Send RPC request 'resume_workflow'[wf_ex_id=%s]",
+            wf_ex_id
+        )
+
         return self._client.sync_call(
             auth_ctx.ctx(),
             'resume_workflow',
@@ -304,6 +367,14 @@ class EngineClient(eng.Engine):
         :return: Workflow execution, model.Execution
         """
 
+        LOG.info(
+            "Send RPC request 'stop_workflow'[execution_id=%s,"
+            " state=%s, message=%s]",
+            wf_ex_id,
+            state,
+            message
+        )
+
         return self._client.sync_call(
             auth_ctx.ctx(),
             'stop_workflow',
@@ -321,6 +392,11 @@ class EngineClient(eng.Engine):
         :return: Workflow execution.
         """
 
+        LOG.info(
+            "Send RPC request 'rollback_workflow'[execution_id=%s]",
+            wf_ex_id
+        )
+
         return self._client.sync_call(
             auth_ctx.ctx(),
             'rollback_workflow',
@@ -333,6 +409,12 @@ class EngineClient(eng.Engine):
 
         :param action_ex_ids: Action execution ids.
         """
+
+        LOG.info(
+            "Send RPC request 'report_running_actions'[action_ex_ids=%s]",
+            action_ex_ids
+        )
+
         return self._client.async_call(
             auth_ctx.ctx(),
             'report_running_actions',
@@ -350,18 +432,15 @@ class ExecutorClient(exe.Executor):
         self._client = base.get_rpc_client_driver()(rpc_conf_dict)
 
     @profiler.trace('executor-client-run-action')
-    def run_action(self, action_ex_id, action_cls_str, action_cls_attrs,
-                   params, safe_rerun, execution_context, redelivered=False,
-                   target=None, async_=True, timeout=None):
+    def run_action(self, action, action_ex_id, safe_rerun, exec_ctx,
+                   redelivered=False, target=None, async_=True, timeout=None):
         """Sends a request to run action to executor.
 
+        :param action: Action to run.
         :param action_ex_id: Action execution id.
-        :param action_cls_str: Action class name.
-        :param action_cls_attrs: Action class attributes.
-        :param params: Action input parameters.
         :param safe_rerun: If true, action would be re-run if executor dies
             during execution.
-        :param execution_context: A dict of values providing information about
+        :param exec_ctx: A dict of values providing information about
             the current execution.
         :param redelivered: Tells if given action was run before on another
             executor.
@@ -372,23 +451,23 @@ class ExecutorClient(exe.Executor):
             action will be interrupted
         :return: Action result.
         """
-
         rpc_kwargs = {
+            'action': action,
             'action_ex_id': action_ex_id,
-            'action_cls_str': action_cls_str,
-            'action_cls_attrs': action_cls_attrs,
-            'params': params,
             'safe_rerun': safe_rerun,
-            'execution_context': execution_context,
+            'exec_ctx': exec_ctx,
             'timeout': timeout
         }
 
-        rpc_client_method = (self._client.async_call
-                             if async_ else self._client.sync_call)
+        rpc_client_method = (
+            self._client.async_call if async_
+            else self._client.sync_call
+        )
 
-        LOG.debug(
-            "Sending an action to executor [action_ex_id=%s, action_cls=%s]",
-            action_ex_id, action_cls_str
+        LOG.info(
+            "Send RPC request 'run_action' [action=%s, action_ex_id=%s]",
+            action,
+            action_ex_id
         )
 
         return rpc_client_method(auth_ctx.ctx(), 'run_action', **rpc_kwargs)
@@ -402,6 +481,10 @@ class EventEngineClient(evt_eng.EventEngine):
         self._client = base.get_rpc_client_driver()(rpc_conf_dict)
 
     def create_event_trigger(self, trigger, events):
+        LOG.info(
+            "Send RPC request 'create_event_trigger'[trigger=%s, "
+            "events=%s", trigger, events
+        )
         return self._client.async_call(
             auth_ctx.ctx(),
             'create_event_trigger',
@@ -411,6 +494,10 @@ class EventEngineClient(evt_eng.EventEngine):
         )
 
     def delete_event_trigger(self, trigger, events):
+        LOG.info(
+            "Send RPC request 'delete_event_trigger'[trigger=%s, "
+            "events=%s", trigger, events
+        )
         return self._client.async_call(
             auth_ctx.ctx(),
             'delete_event_trigger',
@@ -420,6 +507,10 @@ class EventEngineClient(evt_eng.EventEngine):
         )
 
     def update_event_trigger(self, trigger):
+        LOG.info(
+            "Send RPC request 'update_event_trigger'[rpc_ctx=%s,"
+            " trigger=%s", trigger
+        )
         return self._client.async_call(
             auth_ctx.ctx(),
             'update_event_trigger',
