@@ -23,7 +23,6 @@ from mistral.db.v2 import api as db_api
 from mistral import exceptions as exc
 from mistral.services import workbooks as wb_service
 from mistral.services import workflows as wf_service
-from mistral.tests.unit import base as test_base
 from mistral.tests.unit.engine import base
 from mistral.workflow import data_flow
 from mistral.workflow import states
@@ -390,6 +389,12 @@ class WithItemsEngineTest(base.EngineTestCase):
             wf_input={'items': [1, 2, 3]}
         )
 
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_ex = wf_ex.task_executions[0]
+
+        self.await_task_running(task_ex.id)
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -553,7 +558,7 @@ class WithItemsEngineTest(base.EngineTestCase):
                   one_two_three: <% task(task1).result %>
         """
         # Register random sleep action in the DB.
-        test_base.register_action_class('sleep_echo', RandomSleepEchoAction)
+        self.register_action_class('sleep_echo', RandomSleepEchoAction)
 
         wb_service.create_workbook_v2(wb_text)
 
@@ -609,7 +614,7 @@ class WithItemsEngineTest(base.EngineTestCase):
         self.assertIn(task1_ex.published['result'], ['Guy'])
 
     def test_with_items_concurrency_1(self):
-        wf_with_concurrency_1 = """---
+        wf_text = """---
         version: "2.0"
 
         wf:
@@ -623,11 +628,16 @@ class WithItemsEngineTest(base.EngineTestCase):
               concurrency: 1
         """
 
-        wf_service.create_workflows(wf_with_concurrency_1)
+        wf_service.create_workflows(wf_text)
 
         # Start workflow.
         wf_ex = self.engine.start_workflow('wf')
 
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_ex = wf_ex.task_executions[0]
+
+        self.await_task_running(task_ex.id)
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -764,8 +774,24 @@ class WithItemsEngineTest(base.EngineTestCase):
         wf_service.create_workflows(wf_with_concurrency_yaql)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wf', wf_input={'concurrency': '2'})
+        self.engine.start_workflow('wf', wf_input={'concurrency': '2'})
 
+        with db_api.transaction():
+            wf_execs = db_api.get_workflow_executions()
+
+            wf_ex = self._assert_single_item(wf_execs, name='wf')
+
+            task_ex = self._assert_single_item(
+                wf_ex.task_executions,
+                name='task1'
+            )
+
+        self.await_task_error(task_ex.id)
+
+        with db_api.transaction():
+            wf_execs = db_api.get_workflow_executions()
+
+            wf_ex = self._assert_single_item(wf_execs, name='wf')
         self.assertIn(
             'Invalid data type in ConcurrencyPolicy',
             wf_ex.state_info
@@ -794,6 +820,12 @@ class WithItemsEngineTest(base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wf')
 
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_ex = wf_ex.task_executions[0]
+
+        self.await_task_running(task_ex.id)
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -948,7 +980,12 @@ class WithItemsEngineTest(base.EngineTestCase):
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
             task_ex = wf_ex.task_executions[0]
+        self.await_task_running(task_ex.id)
 
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_ex = wf_ex.task_executions[0]
         self.assertEqual(3, self._get_running_actions_count(task_ex.id))
 
         # 1st iteration complete.
