@@ -25,7 +25,7 @@ uncommitted=$(git status --porcelain | grep -v "^??")
 git checkout HEAD^
 
 baseline_report=$(mktemp -t mistral_coverageXXXXXXX)
-find . -type f -name "*.pyc" -delete && python setup.py testr --coverage --testr-args="$*"
+find . -type f -name "*.pyc" -delete && stestr --coverage "$*"
 coverage report -m > $baseline_report
 baseline_missing=$(awk 'END { print $3 }' $baseline_report)
 previous_sha=$(git rev-parse HEAD);
@@ -39,7 +39,7 @@ coverage erase;
 
 # Generate and save coverage report
 current_report=$(mktemp -t mistral_coverageXXXXXXX)
-find . -type f -name "*.pyc" -delete && python setup.py testr --coverage --testr-args="$*"
+find . -type f -name "*.pyc" -delete && stestr --coverage "$*"
 coverage report -m > $current_report
 current_missing=$(awk 'END { print $3 }' $current_report)
 
